"""
Take a screenshot
"""
from argparse import ArgumentParser
from logging import getLogger
from re import search

from screamshot import generate_bytes_img
from screamshot.utils import to_sync, open_browser, close_browser


logger = getLogger('screamshot')


def main():
    parser = ArgumentParser(description=__doc__)

    # Mandatory arguments
    parser.add_argument("url", help="url to screenshot")

    parser.add_argument("path", help="path to image to be written")

    # Optionnal arguments
    parser.add_argument("-f", "--fullpage", action="store_true",
                        help="Take a screenshot of the whole scrollable page")

    parser.add_argument("--width", type=int, metavar="width", default=800,
                        help="Set the width of the page")

    parser.add_argument("--height", type=int, metavar="height", default=600,
                        help="Set the height of the page")

    parser.add_argument("--wait_until", nargs="+", type=str,
                        choices=["load", "domcontentloaded",
                                 "networkidle0", "networkidle2"],
                        default="load",
                        help="How long do you want to wait for the page to be loaded")

    # Credentials group
    credentials_group = parser.add_argument_group(title="Credentials (optional)")
    credentials_group.add_argument('--username', help='The username to use')
    credentials_group.add_argument('--password', help='The password to use')
    credentials_group.add_argument('--token', help='The header line to add. \
        Must be like the following expression: key:token')

    # CSS3 selectors group
    selector_group = parser.add_argument_group(title="CSS3 selectors (optional)",
                                               description="Using quotes is recommended for \
                                                   complex CSS3 selectors")
    selector_group.add_argument("--selector", metavar="seletor",
                                help="The CSS3 selector of the element you want in the screenshot")

    selector_group.add_argument("--wait_for", metavar="wait_for_selector",
                                help="The CSS3 selector of an element you want to wait to be \
                                    loaded before taking the screenshot")

    # Browser group
    browser_group = parser.add_argument_group(title='Browser (optional)',
                                              description='By default a headless browser is \
                                                  opened and closed')
    browser_group.add_argument('--no-browser', action='store_true', default=False,
                               help='No browser is opened')
    browser_group.add_argument('-g', '--graphic', dest="headless", action="store_false",
                               default=True, help="Open the browser in graphic mode")
    browser_group.add_argument("--no-sandbox", action="store_const", const=["--no-sandbox"],
                               default="[]", help="Open the browser without sandbox")
    browser_group.add_argument('--no-close', action='store_true', default=False,
                               help='The browser is not closed when the job is done')

    args = parser.parse_args()

    credentials = None
    if args.username and not args.password:
        logger.error('A password must be specified')
        exit(1)
    elif not args.username and args.password:
        logger.error('A username must be specified')
        exit(1)
    elif args.username and args.password:
        credentials = {'username': args.username, 'password': args.password}
    elif args.token:
        regex_token = search(r'(?P<key>[^<]+)\:(?P<token>[^<]+)', args.token)
        try:
            key = regex_token.group('key')
            token = regex_token.group('token')
            credentials = {key: token, 'token_in_header': True}
        except AttributeError as _:
            logger.error('Bad token argument, please read the documentation')
            exit(1)

    if not args.no_browser:
        to_sync(
            open_browser(args.headless, launch_args=args.no_sandbox))

    to_sync(
        generate_bytes_img(args.url, path=args.path, width=args.width, height=args.height,
                           full_page=args.fullpage, selector=args.selector,
                           wait_for=args.wait_for, wait_until=args.wait_until,
                           credentials=credentials))

    if not args.no_browser and not args.no_close:
        to_sync(close_browser())


if __name__ == '__main__':
    main()



"""
Bytes to png function
"""
from io import BytesIO

from PIL import Image


def bytes_to_png(b_img: bytes, path: str):
    """
    Transform the byte object representing an image into a png file

    :param b_img: the image
    :type b_img: bytes

    :param path: the path to the future png file
    :type path: str
    """
    buffer = BytesIO(b_img)
    img = Image.open(buffer)
    img.save(path)



"""
Screamshot exceptions
"""
class ScreamshotException(Exception):
    """
    Base screamshot exception
    """


class BadUrl(ScreamshotException):
    """
    Bad url excpetion
    """


class BadSelector(ScreamshotException):
    """
    Selector exception
    """



"""
Create or close a browser
"""
from argparse import ArgumentParser

from screamshot.utils import close_browser, open_browser, to_sync


def main():
    parser = ArgumentParser(description=__doc__)

    # Mandatory arguments
    group = parser.add_mutually_exclusive_group()
    group.add_argument("-c", "--close", action="store_true", help="Close the browsers in the \
        endpointlist.txt file")
    group.add_argument("-o", "--open", action="store_true", help="Open a browser and store its \
        websocket endpoint in endpointlist.txt")

    # Optionnal argument
    parser.add_argument("-g", "--graphic", dest="headless", action="store_false", help="Open the \
        browser in graphic mode")

    parser.add_argument("-ns", "--no-sandbox", action="store_const", const=["--no-sandbox"],
                        default="[]", help="Open the browser without sandbox")

    args = parser.parse_args()

    if args.close:
        to_sync(close_browser())
    if args.open:
        to_sync(open_browser(args.headless, launch_args=args.no_sandbox))


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Meta data for screamshot."""


__author__ = """Maxime Courtet & Félix Cloup"""
__version__ = "0.1.10"


from screamshot.generate_bytes_img_functions import (
    generate_bytes_img,
    generate_bytes_img_prom,
    generate_bytes_img_wrap,
)

from screamshot.bytes_to_png_function import bytes_to_png


__all__ = [
    "generate_bytes_img",
    "generate_bytes_img_prom",
    "generate_bytes_img_wrap",
    "bytes_to_png",
]



"""
Collection of functions.
"""
import logging
import asyncio
from os import remove
from os.path import join
from time import sleep
from typing import Any, Optional
from tempfile import gettempdir

from urllib3.exceptions import MaxRetryError

from requests import get
from requests.exceptions import ConnectionError as RequestsConnectionError

from pyppeteer import launch, connect
from pyppeteer.browser import Browser


FILENAME_ENDPOINT = join(gettempdir(), "screamshot_browser_endpoint")


logger = logging.getLogger()

logger.setLevel(logging.WARNING)

formatter = logging.Formatter("%(asctime)s :: %(levelname)s :: %(message)s")

stream_handler = logging.StreamHandler()
stream_handler.setLevel(logging.DEBUG)
logger.addHandler(stream_handler)


def _check_wait_until_arg(wait_until_arg: list) -> bool:
    values = ["load", "domcontentloaded", "networkidle0", "networkidle2"]
    return wait_until_arg in values or all([arg in values for arg in wait_until_arg])


def to_sync(fun: Any) -> Any:
    """
    This synchronous function execute and returns the result of an asynchronous function

    :param fun: mandatory, the asynchronous function
    :type fun: function

    .. warning :: This function uses `asyncio` package
    """
    return asyncio.get_event_loop().run_until_complete(fun)


def url_match(url1: str, url2: str) -> bool:
    """
    This function takes two urls and check if they are the same modulo '/' at the end
    :param url1: mandatory, the first url
    :type url1: str

    :param url2: mandatory, the second url
    :type url2: str

    :rtype: Boolean
    """
    return url1 == url2 or url1[:-1] == url2 or url1 == url2[:-1]


def set_endpoint(ws_endpoint: str):
    """
    This function store the web socket endpoint of the launched browser

    :param ws_endpoint: mandatory, the web socket endpoint
    :type ws_endpoint: str

    .. note :: ws_endpoint is saved in FILENAME_ENDPOINT

    .. warning :: It must have the following form: http://.../..., for get_endpoint to work
    """
    with open(FILENAME_ENDPOINT, "w") as ws_file:
        ws_file.write(ws_endpoint + "\n")


def get_endpoint() -> Optional[str]:
    """
    This function returns the web socket endpoint of the running browser

    :retype: str

    .. note :: ws_endpoint is retrieved from FILENAME_ENDPOINT
    """
    try:
        with open(FILENAME_ENDPOINT, "r") as ws_file:
            line = ws_file.readline()
            line = line.split()[0]
            return line
    except FileNotFoundError:
        logger.warning(str(FILENAME_ENDPOINT + " not found"))
        return None


async def open_browser(
        is_headless: bool, launch_args: list = None, write_websocket: bool = True
) -> Browser:
    """
    Launch a browser and writes its websocket endpoint in FILENAME_ENDPOINT if needed

    :param is_headless: mandatory, should the browser be launched in headless mode ?
    :type is_headless: bool

    :param write_websocket: optional, should we store the websocket endpoint in FILENAME_ENDPOINT ?
    :type write_websocket: bool

    :param launch_args: optional, other optional parameters use
    :type launch_args: list of str

    :return: the opened browser
    :retype: pyppeteer Browser
    """
    browser = await launch(headless=is_headless, autoClose=False, args=launch_args)
    if write_websocket:
        set_endpoint(browser.wsEndpoint)
    return browser


async def close_browser():
    """
    Closes the browser related to ws_endpoint and remove FILENAME_ENDPOINT
    """
    ws_endpoint = get_endpoint()
    if ws_endpoint:
        browser = await connect(browserWSEndpoint=ws_endpoint)
        await browser.close()
        remove(FILENAME_ENDPOINT)


async def get_browser(
        is_headless: bool = True, launch_args: list = None, write_websocket: bool = True
) -> Browser:
    """
    Returns a already created browser if one exists or a new one if not

    :param is_headless: optionnal, should the browser be launched in headless mode ?
    :type is_headless: bool

    :param write_websocket: optional, should we store the websocket endpoint in FILENAME_ENDPOINT ?
    :type write_websocket: bool

    :retype: pyppeteer.browser.Browser
    """
    endpoint = get_endpoint()
    if endpoint:
        return await connect(browserWSEndpoint=endpoint)
    return await open_browser(
        is_headless, launch_args=launch_args, write_websocket=write_websocket
    )


def wait_server_start(url: str, waiting_message: str, final_message: str):
    """
    Wait for a web page to answer

    :param url: the url of the web page
    :type url: str

    :param waiting_message: this message will pop up every minute
    :type waiting_message: str

    :param final_message: this message will pop up when the connection is stable
    :type final_message: str

    .. info :: if you add `%d` in your messages you can include time count in it

    .. warning :: Raise a MaxRetryError if the server has not answered after 10s
    """
    count = 0
    server_started = False
    while not server_started:
        if count == 10:
            raise MaxRetryError(url, None)
        try:
            get(url)
            server_started = True
        except (RequestsConnectionError, MaxRetryError) as _:
            sleep(1)
            logger.info(waiting_message, count)
            count += 1
    logger.info(final_message, count)


def wait_server_close(url: str, waiting_message: str, final_message: str):
    """
    Wait for a server to close

    :param url: the url to close the server
    :type url: str

    :param waiting_message: this message will pop up every minute
    :type waiting_message: str

    :param final_message: this message will pop up when the connection is down
    :type final_message: str

    .. info :: if you add `%d` in your messages you can include time count in it

    .. warning :: Raise a MaxRetryError if the server has not closed after 10s
    """
    count = 0
    server_started = False
    while not server_started:
        if count == 10:
            raise MaxRetryError(url, None)
        try:
            get(url)
            sleep(1)
            logger.info(waiting_message, count)
            count += 1
        except (RequestsConnectionError, MaxRetryError) as _:
            server_started = True
    logger.info(final_message, count)
